Module: Rex::IO::DatagramAbstraction

Defined in:
lib/rex/io/datagram_abstraction.rb

Overview

This class provides an abstraction to a datagram based connection through the use of a datagram socketpair.

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#lsockObject

The left side of the stream (local)



26
27
28
# File 'lib/rex/io/datagram_abstraction.rb', line 26

def lsock
  @lsock
end

#rsockObject

The right side of the stream (remote)



28
29
30
# File 'lib/rex/io/datagram_abstraction.rb', line 28

def rsock
  @rsock
end

Instance Method Details

#initialize_abstractionObject

Creates a streaming socket pair



20
21
22
# File 'lib/rex/io/datagram_abstraction.rb', line 20

def initialize_abstraction
  self.lsock, self.rsock = Rex::Socket.udp_socket_pair()
end