Class: Rex::Post::Thread

Inherits:
Object
  • Object
show all
Defined in:
lib/rex/post/thread.rb

Overview

This class provides generalized methods for interacting with a thread running in a process on a remote machine via a post-exploitation client.

Instance Method Summary collapse

Instance Method Details

#close ⇒ Object

Close resources associated with the thread.



52
53
54
# File 'lib/rex/post/thread.rb', line 52

def close
  raise NotImplementedError
end

#query_regs ⇒ Object

Query architecture-specific register state.



38
39
40
# File 'lib/rex/post/thread.rb', line 38

def query_regs
  raise NotImplementedError
end

#resume ⇒ Object

Resume execution of the remote thread.



24
25
26
# File 'lib/rex/post/thread.rb', line 24

def resume
  raise NotImplementedError
end

#set_regs ⇒ Object

Set architecture-specific register state.



45
46
47
# File 'lib/rex/post/thread.rb', line 45

def set_regs
  raise NotImplementedError
end

#suspend ⇒ Object

Suspend the remote thread.



17
18
19
# File 'lib/rex/post/thread.rb', line 17

def suspend
  raise NotImplementedError
end

#terminate ⇒ Object

Terminate the remote thread.



31
32
33
# File 'lib/rex/post/thread.rb', line 31

def terminate
  raise NotImplementedError
end